from pathlib import Path

import typer
from pl_tiny_clients.git_add_all import git_add_all
from pl_tiny_clients.git_any_uncommitted_changes import git_any_uncommitted_changes
from pl_tiny_clients.git_commit import git_commit
from pl_tiny_clients.git_push import git_push
from pl_user_io.display import display
from pl_user_io.loading_spinner import loading_spinner

from pl_ci_cd import check


def commit(directory: Path | None = None) -> None:
    if directory is None:
        directory = Path.cwd()

    if not git_any_uncommitted_changes():
        display("No changes to commit, exiting.")
        return

    check(fix=True, directory=directory)

    git_add_all()

    git_commit(message="Commit")

    with loading_spinner("Pushing changes"):
        git_push()


def main() -> None:
    typer.run(commit)  # pragma: no cover



from pathlib import Path
from types import NoneType

from pl_mocks_and_fakes import MockInUnitTests, MockReason


@MockInUnitTests(MockReason.UNINVESTIGATED)
def make_file_executble(file_name: str) -> NoneType:
    path = Path(file_name)
    current_permissions = path.stat().st_mode
    path.chmod(current_permissions | 0o100)
    return None



from pl_run_program.run_simple_program import run_simple_program

from ._constants import UV_PROGRAM


def package_version() -> str:
    return run_simple_program(UV_PROGRAM, ["version", "--short"]).strip()



from pl_mocks_and_fakes import THIRD_PARTY_API_MOCK_REASONS, MockInUnitTests
from pl_run_program.run_program import run_program

from ._constants import UV_PROGRAM


@MockInUnitTests(*THIRD_PARTY_API_MOCK_REASONS)
def publish_to_pypi() -> None:
    run_program(UV_PROGRAM, ["publish"])



import contextlib
from pathlib import Path
from typing import Annotated

import typer
from pl_user_io.loading_spinner import loading_spinner

from pl_ci_cd._check_format import check_format
from pl_ci_cd._check_types import check_types
from pl_ci_cd._lint import lint
from pl_ci_cd._run_unit_tests_and_coverage import run_unit_tests_and_coverage


def check(
    directory: Path | None = None,
    fix: Annotated[
        bool,
        typer.Option(
            help="Auto-fix issues where possible. Replaces `ruff format --check` with `ruff format` and adds `--fix` to `ruff check`.",
        ),
    ] = False,
) -> None:
    r"""
    Run all CI checks: format, lint, type check, and tests with coverage.

    The following commands are run in order (<dir> is the current working directory):

    \b
    uv --directory <dir> run ruff format --check
    uv --directory <dir> run ruff check
    uv run pyright <dir>
    uv run pytest --cov --cov-fail-under=100 -q <dir>
    """
    if directory is None:
        directory = Path.cwd()

    pyproject_root = directory
    while not (pyproject_root / "pyproject.toml").exists():
        if pyproject_root.parent == pyproject_root:
            msg = f"Could not find `pyproject.toml` in {directory} or any parent directories."
            raise FileNotFoundError(msg)
        pyproject_root = pyproject_root.parent

    with contextlib.chdir(pyproject_root):
        with loading_spinner("Formatter"):
            check_format(pyproject_root, fix)

        with loading_spinner("Linter"):
            lint_result = lint(pyproject_root, fix=fix)
            if not lint_result.passed:
                msg = f"Linting failed, aborting commit. Output:\n{lint_result.output}"
                raise RuntimeError(msg)

        with loading_spinner("Type checks"):
            check_types(pyproject_root)

        with loading_spinner("Tests + Coverage"):
            run_unit_tests_and_coverage(pyproject_root)


def main() -> None:
    typer.run(check)  # pragma: no cover
